@@ -1,3 +1,6 @@
+from __future__ import annotations
+
+import time
 import typing
 import asyncio
 import logging
@@ -15,8 +18,8 @@ import zigpy_znp.commands as c
 from zigpy_znp import uart
 from zigpy_znp.nvram import NVRAMHelper
 from zigpy_znp.frames import GeneralFrame
+from zigpy_znp.znp.utils import NetworkInfo, load_network_info, detect_zstack_version
 from zigpy_znp.exceptions import CommandNotRecognized, InvalidCommandResponse
-from zigpy_znp.types.nvids import ExNvIds, NvSysIds
 
 LOGGER = logging.getLogger(__name__)
 AFTER_CONNECT_DELAY = 1  # seconds
@@ -25,7 +28,7 @@ STARTUP_DELAY = 1  # seconds
 
 def _deduplicate_commands(
     commands: typing.Iterable[t.CommandBase],
-) -> typing.Tuple[t.CommandBase]:
+) -> tuple[t.CommandBase]:
     """
     Deduplicates an iterable of commands by folding more-specific commands into less-
     specific commands. Used to avoid triggering callbacks multiple times per packet.
@@ -56,32 +59,9 @@ def _deduplicate_commands(
     return tuple(maximal_commands)
 
 
-async def detect_zstack_version(znp) -> float:
-    """
-    Feature detects the major version of Z-Stack running on the device.
-    """
-
-    # Z-Stack 1.2 does not have the AppConfig subsystem
-    if not znp.capabilities & t.MTCapabilities.CAP_APP_CNF:
-        return 1.2
-
-    try:
-        # Only Z-Stack 3.30+ has the new NVRAM system
-        await znp.nvram.read(
-            sys_id=NvSysIds.ZSTACK,
-            item_id=ExNvIds.TCLK_TABLE,
-            sub_id=0x0000,
-        )
-        return 3.30
-    except KeyError:
-        return 3.30
-    except CommandNotRecognized:
-        return 3.0
-
-
 @dataclasses.dataclass(frozen=True)
 class BaseResponseListener:
-    matching_commands: typing.Tuple[t.CommandBase]
+    matching_commands: tuple[t.CommandBase]
 
     def __post_init__(self):
         commands = _deduplicate_commands(self.matching_commands)
@@ -92,7 +72,7 @@ class BaseResponseListener:
         # We're frozen so __setattr__ is disallowed
         object.__setattr__(self, "matching_commands", commands)
 
-    def matching_headers(self) -> typing.Set[t.CommandHeader]:
+    def matching_headers(self) -> set[t.CommandHeader]:
         """
         Returns the set of Z-Stack MT command headers for all the matching commands.
         """
@@ -200,11 +180,26 @@ class ZNP:
         self.version = None
 
         self.nvram = NVRAMHelper(self)
+        self.network_info: NetworkInfo = None
 
     def set_application(self, app):
         assert self._app is None
         self._app = app
 
+    async def load_network_info(self):
+        self.network_info = await load_network_info(self)
+
+    async def reset(self) -> None:
+        """
+        Performs a soft reset within Z-Stack.
+        A hard reset resets the serial port, causing the device to disconnect.
+        """
+
+        await self.request_callback_rsp(
+            request=c.SYS.ResetReq.Req(Type=t.ResetType.Soft),
+            callback=c.SYS.ResetInd.Callback(partial=True),
+        )
+
     @property
     def _port_path(self) -> str:
         return self._config[conf.CONF_DEVICE][conf.CONF_DEVICE_PATH]
@@ -257,6 +252,9 @@ class ZNP:
 
                 # Make sure that our port works
                 self.capabilities = (await self.request(c.SYS.Ping.Req())).Capabilities
+
+                # We need to know how structs are packed to deserialize frames corectly
+                await self.nvram.determine_alignment()
                 self.version = await detect_zstack_version(self)
 
                 LOGGER.debug("Detected Z-Stack %s", self.version)
@@ -355,8 +353,12 @@ class ZNP:
         XXX: Can be called multiple times in a single event loop step!
         """
 
+        if frame.header not in c.COMMANDS_BY_ID:
+            LOGGER.error("Received an unknown frame: %s", frame)
+            return
+
         command_cls = c.COMMANDS_BY_ID[frame.header]
-        command = command_cls.from_frame(frame)
+        command = command_cls.from_frame(frame, align=self.nvram.align_structs)
 
         LOGGER.debug("Received command: %s", command)
 
@@ -399,26 +401,13 @@ class ZNP:
         """
 
         queue = asyncio.Queue()
-        listener = self.callback_for_responses(responses, queue.put_nowait)
+        listener = self.callback_for_responses(responses, callback=queue.put_nowait)
 
         try:
             yield queue
         finally:
             self.remove_listener(listener)
 
-    @contextlib.asynccontextmanager
-    async def capture_responses_once(self, responses):
-        """
-        Captures all matched responses in a queue within the context manager.
-        """
-
-        future, listener = self.wait_for_responses(responses, context=True)
-
-        try:
-            yield future
-        finally:
-            self.remove_listener(listener)
-
     def callback_for_responses(self, responses, callback) -> CallbackResponseListener:
         """
         Creates a callback listener that matches any of the provided responses.
@@ -502,6 +491,8 @@ class ZNP:
                 f"will have no effect"
             )
 
+        frame = request.to_frame(align=self.nvram.align_structs)
+
         # We should only be sending one SREQ at a time, according to the spec
         async with self._sync_request_lock:
             LOGGER.debug("Sending request: %s", request)
@@ -509,7 +500,7 @@ class ZNP:
             # If our request has no response, we cannot wait for one
             if not request.Rsp:
                 LOGGER.debug("Request has no response, not waiting for one.")
-                self._uart.send(request.to_frame())
+                self._uart.send(frame)
                 return
 
             # We need to create the response listener before we send the request
@@ -522,7 +513,7 @@ class ZNP:
                 ]
             )
 
-            self._uart.send(request.to_frame())
+            self._uart.send(frame)
 
             # We should get a SRSP in a reasonable amount of time
             async with async_timeout.timeout(
@@ -545,7 +536,7 @@ class ZNP:
         return response
 
     async def request_callback_rsp(
-        self, *, request, callback, timeout=None, **response_params
+        self, *, request, callback, timeout=None, background=False, **response_params
     ):
         """
         Sends an SREQ, gets its SRSP confirmation, and waits for its real AREQ response.
@@ -558,12 +549,41 @@ class ZNP:
         from the UART and be handled in the same event loop step by ZNP.
         """
 
+        # Every request should have a timeout to prevent deadlocks
         if timeout is None:
             timeout = self._config[conf.CONF_ZNP_CONFIG][conf.CONF_ARSP_TIMEOUT]
 
-        # The async context manager allows us to clean up resources upon cancellation
-        async with self.capture_responses_once([callback]) as callback_rsp:
-            await self.request(request, **response_params)
+        callback_rsp, listener = self.wait_for_responses([callback], context=True)
 
+        if not background:
+            try:
+                async with async_timeout.timeout(timeout):
+                    await self.request(request, **response_params)
+
+                    return await callback_rsp
+            finally:
+                self.remove_listener(listener)
+
+        start_time = time.time()
+
+        # If the SREQ/SRSP pair fails, we must cancel the AREQ listener
+        try:
             async with async_timeout.timeout(timeout):
-                return await callback_rsp
+                request_rsp = await self.request(request, **response_params)
+        except Exception:
+            self.remove_listener(listener)
+            raise
+
+        async def callback_handler(timeout):
+            try:
+                async with async_timeout.timeout(timeout):
+                    await callback_rsp
+            finally:
+                self.remove_listener(listener)
+
+        # If it succeeds, create a background task to receive the AREQ but take into
+        # account the time it took to start the SREQ to ensure we do not grossly exceed
+        # the timeout
+        asyncio.create_task(callback_handler(time.time() - start_time))
+
+        return request_rsp
