@@ -6,7 +6,7 @@ returned by the target. The response message only indicates that the command mes
 was received and executed. The result of the command execution will be conveyed to
 the tester via a callback message interface"""
 
-import typing
+from __future__ import annotations
 
 import zigpy.zdo.types
 
@@ -100,7 +100,16 @@ class GroupIdList(t.LVList, item_type=t.GroupId, length_type=t.uint8_t):
     pass
 
 
-class BindEntryList(t.LVList, item_type=t.BindEntry, length_type=t.uint8_t):
+class BindEntry(t.Struct):
+    """Bind table entry."""
+
+    Src: t.EUI64
+    SrcEp: t.uint8_t
+    ClusterId: t.ClusterId
+    DstAddr: zigpy.zdo.types.MultiAddress
+
+
+class BindEntryList(t.LVList, item_type=BindEntry, length_type=t.uint8_t):
     pass
 
 
@@ -118,7 +127,7 @@ class ChildInfoList(t.LVList, item_type=t.EUI64, length_type=t.uint8_t):
 
 class NullableNodeDescriptor(zigpy.zdo.types.NodeDescriptor):
     @classmethod
-    def deserialize(cls, data: bytes) -> typing.Tuple["NullableNodeDescriptor", bytes]:
+    def deserialize(cls, data: bytes) -> tuple[NullableNodeDescriptor, bytes]:
         if data == b"\x00":
             return cls(), b""
 
@@ -1345,6 +1354,17 @@ class ZDO(t.CommandsBase, subsystem=t.Subsystem.ZDO):
         ),
     )
 
+    # ???
+    ConcentratorInd = t.CommandDef(
+        t.CommandType.AREQ,
+        0xC8,
+        rsp_schema=(
+            t.Param("NWK", t.NWK, "Short address"),
+            t.Param("IEEE", t.EUI64, "IEEE address"),
+            t.Param("PktCost", t.uint8_t, "Packet cost"),
+        ),
+    )
+
     # an indication to inform the host of a device leaving the network
     LeaveInd = t.CommandDef(
         t.CommandType.AREQ,
