@@ -2,21 +2,18 @@ import sys
 import json
 import asyncio
 import logging
-import argparse
 
+import zigpy_znp.types as t
 from zigpy_znp.api import ZNP
 from zigpy_znp.config import CONFIG_SCHEMA
 from zigpy_znp.exceptions import SecurityError, CommandNotRecognized
-from zigpy_znp.types.nvids import NWK_NVID_TABLES, ExNvIds, NvSysIds, OsalNvIds
-from zigpy_znp.tools.common import setup_parser
+from zigpy_znp.types.nvids import NWK_NVID_TABLES, ExNvIds, OsalNvIds
+from zigpy_znp.tools.common import ClosableFileType, setup_parser
 
 LOGGER = logging.getLogger(__name__)
 
 
-async def backup(radio_path: str):
-    znp = ZNP(CONFIG_SCHEMA({"device": {"path": radio_path}}))
-    await znp.connect()
-
+async def nvram_read(znp: ZNP):
     data = {}
     data["LEGACY"] = {}
 
@@ -34,9 +31,11 @@ async def backup(radio_path: str):
                 key = f"{nwk_nvid.name}+{offset}"
 
                 try:
-                    value = await znp.nvram.osal_read(nwk_nvid + offset)
+                    value = await znp.nvram.osal_read(
+                        nwk_nvid + offset, item_type=t.Bytes
+                    )
                 except SecurityError:
-                    LOGGER.error("Read not allowed for %s", key)
+                    LOGGER.warning("Read disallowed for %s", key)
                     continue
                 except KeyError:
                     break
@@ -45,12 +44,11 @@ async def backup(radio_path: str):
                 data["LEGACY"][key] = value.hex()
         else:
             try:
-                value = await znp.nvram.osal_read(nwk_nvid)
+                value = await znp.nvram.osal_read(nwk_nvid, item_type=t.Bytes)
             except KeyError:
-                LOGGER.warning("Read failed for %s", nwk_nvid)
                 continue
             except SecurityError:
-                LOGGER.error("Read not allowed for %s", nwk_nvid)
+                LOGGER.warning("Read disallowed for %s", nwk_nvid)
                 continue
 
             LOGGER.info("%s = %s", nwk_nvid, value)
@@ -64,7 +62,7 @@ async def backup(radio_path: str):
         for sub_id in range(2 ** 16):
             try:
                 value = await znp.nvram.read(
-                    sys_id=NvSysIds.ZSTACK, item_id=nvid, sub_id=sub_id
+                    item_id=nvid, sub_id=sub_id, item_type=t.Bytes
                 )
             except CommandNotRecognized:
                 # CC2531 only supports the legacy NVRAM interface, even on Z-Stack 3
@@ -82,13 +80,19 @@ async def backup(radio_path: str):
 async def main(argv):
     parser = setup_parser("Backup a radio's NVRAM")
     parser.add_argument(
-        "--output", "-o", type=argparse.FileType("w"), help="Output file", default="-"
+        "--output", "-o", type=ClosableFileType("w"), help="Output file", default="-"
     )
 
     args = parser.parse_args(argv)
 
-    obj = await backup(args.serial)
-    args.output.write(json.dumps(obj, indent=4))
+    with args.output as f:
+        znp = ZNP(CONFIG_SCHEMA({"device": {"path": args.serial}}))
+        await znp.connect()
+
+        obj = await nvram_read(znp)
+        znp.close()
+
+        f.write(json.dumps(obj, indent=4) + "\n")
 
 
 if __name__ == "__main__":
